#! /usr/bin/env ocaml
#load "unix.cma"

let max_index =
  let ( * ) = Int64.mul in
  Int64.pred( 256L * 256L * 256L )
;;

let () =
  output_byte stdout 3;  (* octets per positions *)
  output_byte stdout 2;  (* octets per lengths *)
;;

let write_pos pos =
  output_byte stdout (Int64.to_int(pos));
  output_byte stdout (Int64.to_int(Int64.shift_right pos  8));
  output_byte stdout (Int64.to_int(Int64.shift_right pos 16));
;;

let write_len len =
  if len > 65535L then failwith "length overflow";
  output_byte stdout (Int64.to_int(len));
  output_byte stdout (Int64.to_int(Int64.shift_right len 8));
;;

let dump pos =
  if pos > max_index then failwith "index overflow";
  (function
  | '<', None -> (write_pos pos; Some pos)
  | '>', Some prev ->
      let ( - ) = Int64.sub in
      let len = Int64.succ(pos - prev) in
      write_len len;
      (None)
  (* {{{ ... *)

  | '>', None ->
      failwith(Printf.sprintf "open tag missing at offset '%Ld'" pos)

  | '<', Some prev ->
      failwith(Printf.sprintf "tag open twice at offsets '%Ld' and '%Ld'" prev pos)

  (* default *)
  | _, cur -> (cur)
  )
  (* }}} *)
;;

let index_datas ~filename =
  let fd = Unix.openfile filename [Unix.O_RDONLY] 0o640 in
  let ic = Unix.in_channel_of_descr fd in

  let rec loop i prev =
    let c = input_char ic in
    let current = dump i (c,prev) in
 (* prerr_string(Printf.sprintf " %Ld" i); *)
    loop (Int64.succ i) current
  in
  try loop 0L None;
  with End_of_file -> Unix.close fd;
;;

let () =
  index_datas Sys.argv.(1);
;;

(* vim: sw=2 sts=2 ts=2 et fdm=marker
 *)
